@@ -13,12 +13,18 @@ def main():
                    'accepts input as a file or stdin and can write out to '
                    'a file or stdout.')
     parser = argparse.ArgumentParser(prog=prog, description=description)
+    parser.add_argument('-v', '--version', action='store_true',
+                        help='show the version number and exit')
     parser.add_argument('infile', nargs='?', type=argparse.FileType(),
                         help='a textile file to be converted')
     parser.add_argument('outfile', nargs='?', type=argparse.FileType('w'),
                         help='write the output of infile to outfile')
     options = parser.parse_args()
 
+    if options.version:
+        print(textile.VERSION)
+        sys.exit()
+
     infile = options.infile or sys.stdin
     outfile = options.outfile or sys.stdout
     with infile:
