@@ -10,10 +10,7 @@ from six.moves import urllib, html_parser
 urlparse = urllib.parse.urlparse
 HTMLParser = html_parser.HTMLParser
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 
 from xml.etree import ElementTree
 
@@ -50,6 +47,8 @@ def generate_tag(tag, content, attributes=None):
     content are strings, the attributes argument is a dictionary.  As
     a convenience, if the content is ' /', a self-closing tag is generated."""
     content = six.text_type(content)
+    # In PY2, ElementTree tostringlist only works with bytes, not with
+    # unicode().
     enc = 'unicode'
     if six.PY2:
         enc = 'UTF-8'
@@ -60,21 +59,12 @@ def generate_tag(tag, content, attributes=None):
     # adding text by assigning it to element_tag.text.  That results in
     # non-ascii text being html-entity encoded.  Not bad, but not entirely
     # matching php-textile either.
-    try:
-        element_tag = ElementTree.tostringlist(element, encoding=enc,
-                method='html')
-        element_tag.insert(len(element_tag) - 1, content)
-        element_text = ''.join(element_tag)
-    except AttributeError:
-        # Python 2.6 doesn't have the tostringlist method, so we have to treat
-        # it differently.
-        element_tag = ElementTree.tostring(element, encoding=enc)
-        element_text = re.sub(r"<\?xml version='1.0' encoding='UTF-8'\?>\n",
-                '', element_tag)
-        if content != six.text_type(' /'):
-            element_text = element_text.rstrip(' />')
-            element_text = six.text_type('{0}>{1}</{2}>').format(six.text_type(
-                element_text), content, tag)
+    element_tag = ElementTree.tostringlist(element, encoding=enc,
+            method='html')
+    if six.PY2:
+        element_tag = [v.decode(enc) for v in element_tag]
+    element_tag.insert(len(element_tag) - 1, content)
+    element_text = ''.join(element_tag)
     return element_text
 
 def has_raw_text(text):
@@ -110,9 +100,8 @@ def list_type(list_string):
 
 def normalize_newlines(string):
     out = string.strip()
-    out = re.sub(r'\r\n', '\n', out)
-    out = re.sub(r'\n{3,}', '\n\n', out)
-    out = re.sub(r'\n\s*\n', '\n\n', out)
+    out = re.sub(r'\r\n?', '\n', out)
+    out = re.compile(r'^[ \t]*\n', flags=re.M).sub('\n', out)
     out = re.sub(r'"$', '" ', out)
     return out
 
